/**
 * overlay.js
 * Copyright (C) 2007-2011 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

org.mozdev.popomungo.classes.Overlay = function Overlay() {};

org.mozdev.popomungo.classes.Overlay.prototype = {
		
	requestCount: 0,
	
	listenerEnabled: false,
	
	contextMenuListenerEnabled: false,
	
	ignoredPages: {
		'cheater': true,
		'cn': true,
		'defaultconf': true,
		'defaultconfv2': true,
		'forumtop': true,
		'gazette': true,
		'menuforum': true,
		'pageexpired': true,
		'performancelive': true,
		'wallpaper': true
	},
	

	/**
	 * Process a game page.
	 *  
	 * @param {Document} aDocument
	 */
	handleContent: function handleContent(aDocument) {
		try {
			this.requestCount++;
			
			var timer1 = new org.mozdev.popomungo.classes.Timer("Request #"+ this.requestCount);
			var path1 = aDocument.location.pathname;
			var query1 = aDocument.location.search;

			org.mozdev.popomungo.logger.debug("Request #"+ this.requestCount +
				"\nhref="+ aDocument.location.href +
				"\npath="+ path1 +
				"\nquery="+ query1);

			// Remove frames
			if (path1 == '/common/index.html' && 
				org.mozdev.popomungo.logger.getTraceLevel()) 
			{
				var frames = aDocument.getElementsByTagName('frame');

				if (frames.length > 0) {
					var frame1 = frames[0];
					
					if (frame1.src) {
						org.mozdev.popomungo.logger.debug('Redirecting to '+ frame1.src);
						aDocument.location.replace(frame1.src);
					}
				}
				return;
			}

			org.mozdev.popomungo.ElementCache.clear();

			var page, action, action2;
			var result = path1.match(/\/([a-z0-9]+)\.asp/i);

			if (result) {
				page = result[1].toLowerCase();
			}
			
			result = query1.match(/[?&]action=([a-z0-9]+)/i);
			if (result) {
				action = result[1].toLowerCase();
			}

			result = query1.match(/[?&]action2=([a-z0-9]+)/i);
			if (result) {
				action2 = result[1].toLowerCase();
			}

			org.mozdev.popomungo.logger.debug("Request #"+ this.requestCount +
				"\npage="+ page +
				"\naction="+ action +
				"\naction2="+ action2);

			var processScoreNumbersEnabled = true;
			var processLinksEnabled = true;
	
			if (page == undefined) {
				org.mozdev.popomungo.logger.debug('Page undefined.');
				return;
			}
			else if (page == 'entry') {
				if (action == 'logout' ||
					action == 'sessiontimeout') 
				{
					org.mozdev.popomungo.UserSettings.LoggedOut(aDocument);
					org.mozdev.popomungo.logger.debug('Logged out.');
					return;
				}
				else if (action == undefined) {
					org.mozdev.popomungo.logger.debug('Not logged in.');
					return;
				}
			}
			else if (page == 'credits') {
				org.mozdev.popomungo.UserSettings.LoggedOut(aDocument);
				org.mozdev.popomungo.logger.debug('Logged out.');
				return;
			}
			else if (this.ignoredPages[page]) {
				org.mozdev.popomungo.logger.debug('Page ignored.');
				return;				
			}
			else if (page == 'choosecharacter') {
				if (org.mozdev.popomungo.UserSettings.charId) {
					org.mozdev.popomungo.UserSettings.LoggedOut(aDocument);
				}
				
				org.mozdev.popomungo.styles.appendCSS(aDocument);
				org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				return;
			}

			var charId = org.mozdev.popomungo.UserSettings.getCharacterId(aDocument);
		
			if (charId) {
				// Save the current character. 
				org.mozdev.popomungo.prefs.setPref("extensions.popomungo.currentCharacter", charId);
				
				var langId = org.mozdev.popomungo.UserSettings.autoDetectLanguageFromPage(aDocument);
				
				org.mozdev.popomungo.strings.InitStringBundle(langId);
				
				org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

				org.mozdev.popomungo.logger.debug("Pre-tasks");

				org.mozdev.popomungo.styles.appendCSS(aDocument);
		
				org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

				org.mozdev.popomungo.logger.debug("Main-tasks");

				if (page == 'city') {
					if (action == 'view' ||
						action == undefined) 
					{
						org.mozdev.popomungo.locales.addLocaleIcons(aDocument);
						org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

						//org.mozdev.popomungo.UserSettings.updateSettingsFromCityPage(aDocument);
						org.mozdev.popomungo.traveling.addRouteInfoOnCityPage(aDocument);
						org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

						// Too many links here, we will need to use another method.
						processScoreNumbersEnabled = false;
						org.mozdev.popomungo.scoring.addNumericScoresOnCityPage(aDocument);
						org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

						processLinksEnabled = false;
						org.mozdev.popomungo.links.addEventHandlerOnCityPage(aDocument);
					}
					// online onlinesamelanguage newbies 
					// ongoingshows topchart viewfriends 
					// bookmarkedlocales
				}
				else if (page == 'user') {
					if (action == 'update') {
						org.mozdev.popomungo.UserSettings.updateSettingsOnLoad(aDocument);
					}
				}
				else if (page == 'performances') {
					if (action == undefined) {
						action = org.mozdev.popomungo.performances.identifyUndefinedPerformancesPage(aDocument);
					}
					
					if (action == 'arrangements') {
						org.mozdev.popomungo.performances.processArrangements(aDocument);
					} else if (action == 'readperformance') {
						org.mozdev.popomungo.performances.addTicketSalesStatistics(aDocument);
					} else if (action == 'citiesplayed') {
						org.mozdev.popomungo.performances.addSoldTicketsOnCitiesPage(aDocument);
					} else if (action == 'viewperformances') {
						org.mozdev.popomungo.performances.addSoldTicketsOnShowsPage(aDocument, false);
					} else if (action == 'clubcancelshow') {
						org.mozdev.popomungo.performances.processArrangements(aDocument);
					} else {
						// comingperformances clearallsetlists
						// accept confirmdiscardp setcancelperformance 
						org.mozdev.popomungo.performances.addSoldTicketsOnShowsPage(aDocument, true);
						org.mozdev.popomungo.Company.highlightCeoLocale(aDocument);
					}
				}
				else if (page == 'performancedetails') {
					// nextshow defaultsetlist viewperformance addsong
					// addaction removeaction
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument, org.mozdev.popomungo.progressBars.WITH_MINUS_SIGN_REGEXP);
					
					processLinksEnabled = false;
				}
				else if (page == 'company') {
					if (action == 'shows' && 
						!action2) 
					{
						org.mozdev.popomungo.progressBars.addTextOverTicketSalesBars(aDocument);
						org.mozdev.popomungo.performances.addTicketSalesTotalOnClubShowsPage(aDocument);
					} else if (action == 'restock') {
						org.mozdev.popomungo.products.highlightEmptyStockOnRestock(aDocument);
					} else if (action == 'manageproducts' ||
						action == 'deleteproductconfirm') 
					{
						org.mozdev.popomungo.products.highlightEmptyStockOnManageProducts(aDocument);
					} else if (action == undefined) {
						org.mozdev.popomungo.products.highlightEmptyStockIfFormPage(aDocument);
					}
					// SetBidLocale
				}
				else if (page == 'characterdetails') {
					if (action == undefined) {
						action = org.mozdev.popomungo.progressBars.identifyUndefinedCharacterDetailsPage(aDocument);
					}

					org.mozdev.popomungo.UserSettings.updateSettingsFromCharacterPage(aDocument);
					org.mozdev.popomungo.traveling.replaceWithLocalizedRouteNameOnPage(aDocument);

					if (action == 'view' || 
						action == undefined) 
					{
						org.mozdev.popomungo.Comparison.processCharacterSheet(aDocument);
					}
					else if (action == 'buddylist' ||
						action == 'deletefriend') 
					{
						org.mozdev.popomungo.Friends.highlightCityOnBuddyList(aDocument);
					}
					else if (action == 'relations' ||
						action == "abandonattachment") 
					{
						org.mozdev.popomungo.Friends.highlightCityOnRelationsPage(aDocument);
					}
					
					if (action == 'myskills' ||
						action == 'selectskill') {
						org.mozdev.popomungo.progressBars.addTextOverStats(aDocument, org.mozdev.popomungo.progressBars.PERCENTAGE_REGEXP);
					}
					else if (action == "apprentices" ||
						action == "removemaster" ||
						action == "removeapprentice") {
						org.mozdev.popomungo.progressBars.addTextOverStats(aDocument, org.mozdev.popomungo.progressBars.WITH_DECIMALS_REGEXP);
					}
					else {
						org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					}

					// viewitems wear takeoff drop undress distributeexp
					// setpanda +myskills Influences ViewAssets Miscellaneous 
					// Photos Achievements CharacterPic Messages Calendar 
					// CurrencyConverter WorldClock readmessage AnswerMessage
					// +SelectSkill
				}
				else if (page == 'telephone') {
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					org.mozdev.popomungo.Friends.highlightCityOnPhoneBook(aDocument);

					// undefined phonebook SendMessage OnlineShopping BookMarks 
					// NotePad Instructions FillCash Dial DeleteBM
				}
				else if (page == 'tribes') {
					if (action == 'members') {
						org.mozdev.popomungo.Friends.highlightCityOnTribeMembersList(aDocument);
					}
					
					// undefined TribeInfo Competitions Charts
				}
				else if (page == 'interact') {
					//org.mozdev.popomungo.Interaction.sortInteractionChoises(aDocument);
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'relationshipdetails') {
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'artist') {
					if (action == undefined ||
						action == 'view')
					{
						org.mozdev.popomungo.traveling.replaceWithLocalizedRouteNameOnPage(aDocument);
						org.mozdev.popomungo.traveling.addCityNameToBusStationName(aDocument);
					}

					// viewrepertoire popularity stagenames
					// setrepertoirepractise viewartistsong confirmdiscardas
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument, org.mozdev.popomungo.progressBars.WITH_MINUS_SIGN_REGEXP);
 				}
				else if (page == 'bookings') {
					if (action == undefined ||
						action == 'setdelete')
					{
						org.mozdev.popomungo.performances.addSoldTicketsOnSchedulePage(aDocument);
					}
					// QuickBook, SearchBookings, SearchJams, SearchStudio, Delete
				}
				else if (page == 'song') {
					// viewunfinished view viewsong deletesongconfirmed
					// setworkonsong undefined
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'track') {
					// undefined viewtrack
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'toursettings') {
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					
					if (action == 'transport' ||
						action == 'addtransport' ||
						action == 'canceltransport') 
					{
						org.mozdev.popomungo.traveling.addTravelTimesOnTransportPage(aDocument);
						org.mozdev.popomungo.traveling.replaceWithLocalizedRouteNameOnPage(aDocument);
					}
					else if (action == 'equipment') {
						processLinksEnabled = false;
					}
				}
				else if (page == 'artistranking') {
					// undefined
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'movies') {
					// viewmovie
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'locale') {
					// cmd=confirmexchange (tour bus exchange)
					// viewshop useskills viewcharacters viewworkers
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					if (action == 'view' || 
						action == undefined) 
					{
						org.mozdev.popomungo.traveling.addRouteInfoOnHighwayLocalePage(aDocument);
					}					
				}
				else if (page == 'competition') {
					if (action == 'schedule' ||
						action == 'deleteinvite' ||
						action == 'addopeninvite' ||
						action == 'setinviteartist')
					{
						org.mozdev.popomungo.Competitions.colorizeCompetitionEntries(aDocument);
					}
				}
				else if (page == 'distributeexp') {
					org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
				}
				else if (page == 'charts') {
					if (action == 'paparazzilist') {
						org.mozdev.popomungo.Friends.highlightCityOnPaparazziList(aDocument);
					}
					else if (action == 'topstarquality' ||
						action == 'artistrankings' ||
						action == 'topgenres') 
					{
						org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					}
				}
				else if (page == 'charactervehicles')
				{
					if (action == 'viewvehicle' ||
						action == 'transferitem' ||
						action == 'pickupitem')
					{
						org.mozdev.popomungo.progressBars.addTextOverStats(aDocument);
					}
				}
				else {
					org.mozdev.popomungo.logger.debug("No actions");
				}
				// CharacterDiary/view
				// CharacterBlog/ReadBlog CreateBlogEntry

				org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

				org.mozdev.popomungo.logger.debug("Post-tasks");

				org.mozdev.popomungo.bookmarks.showBookmarks(aDocument);
				org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());

				if (processScoreNumbersEnabled) {
					org.mozdev.popomungo.scoring.addNumericScores(aDocument);
					org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());
				}
			
				if (processLinksEnabled) {
					org.mozdev.popomungo.links.processLinks(aDocument);
					org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());
				}
			}
			else {
				org.mozdev.popomungo.logger.debug("Character ID is not available.");
			}
			
			org.mozdev.popomungo.logger.debug(timer1.elapsedTimeStr());
			org.mozdev.popomungo.logger.debug('Request #'+ this.requestCount + ' completed.');
		}
		catch (err)
		{
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	

	isPopmundoDomain: function isPopmundoDomain(aDocument)
	{
		if (aDocument.location && 
			aDocument.location.protocol != 'about:')
		{
			return (null != aDocument.location.hostname.match(org.mozdev.popomungo.commonConstants.POPMUNDO_DOMAIN_REGEXP));
		}
		
		return false;
	},
	

	isPageLoadError: function (aDocument) {
		return (aDocument.documentURI.substr(0,14) == "about:neterror"); 		
	},


	onPageLoad: function onPageLoad(event) {
		var vDocument = event.originalTarget;

 		org.mozdev.popomungo.logger.debug("Target="+ vDocument);
 		
		if (vDocument instanceof HTMLDocument) {
			org.mozdev.popomungo.logger.debug("URL="+ vDocument.URL);		

			
			if (this.isPopmundoDomain(vDocument)) {
				if(!this.isPageLoadError(vDocument)) {
					this.handleContent(vDocument);
				}
				else {
		      		org.mozdev.popomungo.logger.debug("Page load error. Content ignored.");
				}
	      	}
	      	else {
	      		org.mozdev.popomungo.logger.debug("Domain ignored.");
	      	}
		}
		else {
			org.mozdev.popomungo.logger.debug("Target ignored.");
		}
	},

	
	handleEvent: function handleEvent(event) {
		org.mozdev.popomungo.logger.setTraceLevelFromPref();
		org.mozdev.popomungo.logger.debug("Event="+ event.type);
		
		switch (event.type) {
			case "DOMContentLoaded": 
				this.onPageLoad(event);
				break;
			case "load":
				this.init();
				break;
			case "unload":
				this.deinit();
				break;
			default: 
				org.mozdev.popomungo.logger.logError("Unexpected event: "+ event.type);
		}
	},


	init: function init() {
		this.logExtensionInfo();
		
		this.refreshUI();
		
		window.removeEventListener("load", this, false);

		org.mozdev.popomungo.prefs.addObserver("extensions.popomungo.enabled", this);
		org.mozdev.popomungo.prefs.addObserver("extensions.popomungo.traceLevel", this);
		
		org.mozdev.popomungo.prefs.addObserver("extensions.popomungo.options.scoring.colorNumber.gradientColor.from", org.mozdev.popomungo.scoring);
		org.mozdev.popomungo.prefs.addObserver("extensions.popomungo.options.scoring.colorNumber.gradientColor.to", org.mozdev.popomungo.scoring);
		
		org.mozdev.popomungo.prefs.addObserver("extensions.popomungo.characters.", org.mozdev.popomungo.UserSettings);
	},
	
	
	deinit: function deinit() {
		org.mozdev.popomungo.logger.debug("Unload extension.");
		
		org.mozdev.popomungo.prefs.removeObserver("extensions.popomungo.enabled", this);
		org.mozdev.popomungo.prefs.removeObserver("extensions.popomungo.traceLevel", this);
		
		org.mozdev.popomungo.prefs.removeObserver("extensions.popomungo.options.scoring.colorNumber.gradientColor.from", org.mozdev.popomungo.scoring);
		org.mozdev.popomungo.prefs.removeObserver("extensions.popomungo.options.scoring.colorNumber.gradientColor.to", org.mozdev.popomungo.scoring);
		
		org.mozdev.popomungo.prefs.removeObserver("extensions.popomungo.characters.", org.mozdev.popomungo.UserSettings);
		
		document.getElementById("appcontent").removeEventListener("DOMContentLoaded", this, true);

		window.removeEventListener("unload", this, false);
	},
	
	
	dateFormatSettingIsOutOfDate: function dateFormatSettingIsOutOfDate(dateFormatTypeId) {
		if (dateFormatTypeId) {
			org.mozdev.popomungo.prefs.setPref("extensions.popomungo.characters.<n>.dateFormatTypeId", dateFormatTypeId);
		}
	},
	
	
	logExtensionInfo: function logExtensionInfo() {
		try {
			var managerClass = Components.classes["@mozilla.org/extensions/manager;1"];
			
			if (managerClass)
			{
				var manager = managerClass.getService(Components.interfaces.nsIExtensionManager);
		
				var item = manager.getItemForID(org.mozdev.popomungo.commonConstants.GUID); //nsIUpdateItem
				var loc = manager.getInstallLocation(org.mozdev.popomungo.commonConstants.GUID); //nsIInstallLocation
				var canAccess = loc.canAccess ? " (writable)" : " (read-only)";
	
				org.mozdev.popomungo.logger.log(item.name +" "+ item.version);
				org.mozdev.popomungo.logger.debug( 
					"\nGUID: "+ item.id +
					"\nUpdateURL: "+ item.updateRDF +
					"\nTargetApp: "+ item.targetAppID +" "+ item.minAppVersion +"-"+ item.maxAppVersion +
					"\nInstallLoc: "+ loc.name + ", "+ loc.location.path + canAccess);
			}
			else {
				Components.utils.import("resource://gre/modules/AddonManager.jsm");
				
				if (AddonManager) {
					AddonManager.getAddonByID(org.mozdev.popomungo.commonConstants.GUID, this.logExtensionInfoCallback);
				}
				else {
					org.mozdev.popomungo.logger.logError("Could not log extension info.");
				}
			}
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	
	
	logExtensionInfoCallback: function logExtensionInfoCallback(addon) {
		try {
			org.mozdev.popomungo.logger.log(addon.name +" "+ addon.version);
			org.mozdev.popomungo.logger.debug( 
				"\nGUID: "+ addon.id +
				"\nsourceURI: "+ addon.sourceURI.prePath + addon.sourceURI.path +
				"\ninstallDate: "+ addon.installDate +
				"\nupdateDate: "+ addon.updateDate);
		}
		catch (err) {
			org.mozdev.popomungo.logger.logError(err);
		}
	},
	
	
	refreshUI: function refreshUI() {
		org.mozdev.popomungo.logger.setTraceLevelFromPref();
		var enabled = org.mozdev.popomungo.prefs.getPref("extensions.popomungo.enabled", true);

		this.setListener(enabled);
		org.mozdev.popomungo.MenuHandler.setContextMenuListener(enabled);
		this.setStatusbarIcon(enabled);
		this.broadcastState(enabled);

		return enabled;
	},

	
	toggleEnabled: function toggleEnabled() {
		// Toggle state.
		var enabled = !org.mozdev.popomungo.prefs.getPref("extensions.popomungo.enabled", true);
		
		// This triggers observe function.
		org.mozdev.popomungo.prefs.setPref("extensions.popomungo.enabled", enabled);
		
		return enabled;
	},
	

	setListener: function setListener(aEnabled) {
		var appcontent = document.getElementById("appcontent");
		
		if (appcontent) {
			if (aEnabled && !this.listenerEnabled) {
				this.listenerEnabled = true;
				org.mozdev.popomungo.logger.debug("Add DOMContentLoaded listener");
				appcontent.addEventListener("DOMContentLoaded", this, true);
			} else if (!aEnabled && this.listenerEnabled) {
				this.listenerEnabled = false;
				org.mozdev.popomungo.logger.debug("Remove DOMContentLoaded listener");
				appcontent.removeEventListener("DOMContentLoaded", this, true);				
			} else {
				org.mozdev.popomungo.logger.debug('DOMContentLoaded listener enabled='+ this.listenerEnabled);
			}			
		}
	},

	
	onCommand_OptionsMenuitem: function onCommand_OptionsMenuitem() {
		var url = "chrome://popomungo/content/optionsDialog/optionsDialog.xul";
		var features = "alwaysRaised,centerscreen,chrome,titlebar,toolbar,resizable"; // not modal
		
		window.openDialog(url, "PopomungoOptionsDialog", features);
	},
	
	
	onCommand_ToggleEnabledMenuitem: function onCommand_ToggleEnabledMenuitem() {
		this.toggleEnabled();
		return true;
	},
	
	
	onCommand_OpenPopmundoHomepage: function onCommand_OpenPopmundoHomepage(aEvent) {
		org.mozdev.popomungo.logger.debug('Play the game, event='+ aEvent);
		openUILink(org.mozdev.popomungo.commonConstants.GAME_HOMEPAGE, aEvent);
	},


	onCommand_OpenPopomungoHomepage: function onCommand_OpenPopomungoHomepage(aEvent) {
		org.mozdev.popomungo.logger.debug('Visit home page, event='+ aEvent);
		openUILink(org.mozdev.popomungo.commonConstants.EXTENSION_HOMEPAGE, aEvent, false, true);
	},


	onClick_statusbarpanel: function onClick_statusbarpanel(aEvent) {
		org.mozdev.popomungo.logger.debug('button='+ aEvent.button + ', detail='+ aEvent.detail);
		
		if (aEvent.detail == 2) {
			document.getElementById('menupopup_PopomungoStatusBar').hidePopup();
	
			switch (aEvent.button) {
			case 0:
				// Toggle enabled/disabled state
				this.toggleEnabled();
				break;
				
			case 2:
				// Launch options dialog
				//org.mozdev.popomungo.overlay.onCommand_OptionsMenuitem();
				break;
			}
		}
						
		return true;
	},


	setStatusbarIcon: function setStatusbarIcon(aEnabled) {
		// Get status bar panel.
		var sbPanel = document.getElementById("statusbarpanel_Popomungo");
		
		// Set icon.
		if (aEnabled) {
			sbPanel.src = "chrome://popomungo/skin/popomungo-logo-16x16.png";
			sbPanel.tooltipText = sbPanel.getAttribute('_enabledtooltiptext');
		}
		else {
			sbPanel.src = "chrome://popomungo/skin/popomungo-logo-16x16-gray.png";
			sbPanel.tooltipText = sbPanel.getAttribute('_disabledtooltiptext');
		}
	},
	
	
	broadcastState: function broadcastState(aEnabled) {
		var broadcaster = document.getElementById("popomungoBc_IsDisabled");
		broadcaster.setAttribute('checked',  !aEnabled);
		broadcaster.setAttribute('disabled', !aEnabled);
		broadcaster.setAttribute('hidden',   !aEnabled);
	},
	
	
	/**
	 * Observe preferences.
	 * 
	 * @param aSubject
	 * @param aTopic
	 * @param aData
	 */
	observe: function observe(aSubject, aTopic, aData)
	{
		org.mozdev.popomungo.logger.debug('topic='+ aTopic +', data='+ aData);
		
		if (aTopic == 'nsPref:changed') {
			this.refreshUI();
		}
	}	
};

org.mozdev.popomungo.overlay = new org.mozdev.popomungo.classes.Overlay();

window.addEventListener("unload", org.mozdev.popomungo.overlay, false);
window.addEventListener("load", org.mozdev.popomungo.overlay, false);

///////////////////////////////////////////////////////////////////////
// EOF
///////////////////////////////////////////////////////////////////////